/*
 * png2c - converts a PNG to C source
 *
 * (c) 2001-2002 by David Gerber <zapek@meanmachine.ch>
 */

#include <stdio.h>
#include <png.h>

#define ID1 "$I"
#define ID2 "d$"

#define FMT_ARGB 1
#define FMT_RGB  2

int main(int argc, char **argv)
{
	if (argc == 3 || argc == 4)
	{
		FILE *in, *out, *hout;

		if (in = fopen(argv[1], "rb"))
		{
			if (argc == 4)
			{
				if (!(hout = fopen(argv[3], "w")))
				{
					printf("unable to open hfile\n");
					return (0);
				}
			}

			if (out = fopen(argv[2], "w"))
			{
				char header[8];

				fread(header, 1, 8, in);
				if (!png_sig_cmp(header, 0, 8))
				{
					png_structp png_ptr;

					if (png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL))
					{
						png_infop info_ptr;
						png_infop end_info = NULL;

						if (info_ptr = png_create_info_struct(png_ptr))
						{
							if (end_info = png_create_info_struct(png_ptr))
							{
								int supported = 0;

								if (setjmp(png_jmpbuf(png_ptr)))
								{
									printf("fatal error\n");
									png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
									return (0);
								}
								png_init_io(png_ptr, in);
								
								png_set_sig_bytes(png_ptr, 8);
								png_set_filler(png_ptr, 0x00, PNG_FILLER_BEFORE); /* for RGB */
								
								png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_PACKING | PNG_TRANSFORM_SWAP_ALPHA, NULL);

								printf("type is: ");
								switch (info_ptr->color_type)
								{
									case PNG_COLOR_TYPE_GRAY:
										printf("gray\n");
										break;

									case PNG_COLOR_TYPE_PALETTE:
										printf("palette based\n");
										break;

									case PNG_COLOR_TYPE_RGB:
										printf("RGB\n");
										supported = FMT_RGB;
										break;

									case PNG_COLOR_TYPE_RGB_ALPHA:
										printf("RGBA\n");
										supported = FMT_ARGB;
										break;
	
									case PNG_COLOR_TYPE_GRAY_ALPHA:
										printf("gray with alpha\n");
										break;

									default:
										printf("unknown color type\n");
								}

								printf("width: %ld, height: %ld, bpp: %ld\n", info_ptr->width, info_ptr->height, info_ptr->pixel_depth);

								if (supported)
								{
									char *p;
									char filename[108];
									int lines;
									int i;
									int fmt = 0;
									unsigned long **row_ptr;

									row_ptr	= (unsigned long **)png_get_rows(png_ptr, info_ptr);

									if (p = strrchr(argv[1], '/'))
									{
										p++;
									}
									else if (p = strchr(argv[1], ':'))
									{
										p++;
									}
									else
									{
										p = argv[1];
									}

									stccpy(filename, p, 108);
									if (p = strchr(filename, '.'))
									{
										*p = '\0';
									}

									fputs("/*\n * Automatically generated by png2c\n *\n * "ID1 ID2"\n */\n\n", out);
									if (supported == FMT_RGB)
									{
										fprintf(out, "unsigned char %s[] = {\n\t", filename);
									}
									else
									{
										fprintf(out, "unsigned long %s[] = {\n\t", filename);
									}

									for (lines = 0; lines < info_ptr->height; lines++)
									{
										for (i = 0; i < info_ptr->width; i++)
										{
											if (fmt == info_ptr->width)
											{
												fputs("\n\t", out);
												fmt = 0;
											}
											if (supported == FMT_RGB)
											{
												fprintf(out, "0x%02lx, 0x%02lx, 0x%02lx, ", *((unsigned char *)(*row_ptr + i) + 1), *((unsigned char *)(*row_ptr + i) + 2), *((unsigned char *)(*row_ptr + i) + 3));
											}
											else
											{
												fprintf(out, "0x%08lx, ", *(*row_ptr + i));
											}
											fmt++;
										}
										row_ptr++;
									}
									fputs("\n};\n", out);

									if (argc == 4)
									{
										fputs("/*\n * Automatically generated by png2c\n *\n * "ID1 ID2"\n */\n\n", hout);
										if (supported == FMT_RGB)
										{
											fprintf(hout, "\nextern unsigned char %s[];\n", filename);
										}
										else
										{
											fprintf(hout, "\nextern unsigned long %s[];\n", filename);
										}

										p = filename;
										while (*p)
										{
											*p = toupper(*p);
											p++;
										}
										fprintf(hout, "#define %s_WIDTH %ld\n", filename, info_ptr->width);
										fprintf(hout, "#define %s_HEIGHT %ld\n", filename, info_ptr->height);
										fprintf(hout, "#define %s_DEPTH %ld\n", filename, (supported == FMT_RGB) ? 24 : 32);
									}
									printf("processing finished\n");
	
								}
								else
								{
									printf("color type not supported\n");
								}
							}
							else
							{
								printf("couldn't create PNG end info\n");
							}
						}
						else
						{
							printf("couldn't create PNG info\n");
						}

						if (end_info)
						{
							png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
						}
						else if (info_ptr)
						{
							png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
						}
						else
						{
							png_destroy_read_struct(&png_ptr, NULL, NULL);
						}
					}
					else
					{
						printf("couldn't create PNG structure\n");
					}
				}
				else
				{
					printf("not a PNG file\n");
				}
				fclose(out);
			}
			else
			{
				printf("cannot open outfile\n");
			}
			if (argc == 4)
			{
				fclose(hout);
			}
			fclose(in);
		}
	}
	else
	{
		if (argc == 2 && !strcasecmp("-v", argv[1]))
		{
			printf("png2c 1.1\n(c) 2001-2002 by David Gerber <zapek@meanmachine.ch>\nFreeware - All Rights Reserved.\n");
		}
		else
		{
			printf("usage: %s <pngfile> <cfile> [hfile]\n       -v for version info\n", argv[0]);
		}
	}

	return (0);
}
